@@ -11,6 +11,7 @@ from ...metaApi.models import MetatraderHistoryOrders, MetatraderDeals, date, ra
 from .latencyListener import LatencyListener
 from .packetOrderer import PacketOrderer
 from .packetLogger import PacketLogger
+from .synchronizationThrottler import SynchronizationThrottler
 import socketio
 import asyncio
 import re
@@ -35,15 +36,24 @@ class MetaApiWebsocketClient:
         self._url = f'https://mt-client-api-v1.{opts["domain"] if "domain" in opts else "agiliumtrade.agiliumtrade.ai"}'
         self._request_timeout = opts['requestTimeout'] if 'requestTimeout' in opts else 60
         self._connect_timeout = opts['connectTimeout'] if 'connectTimeout' in opts else 60
+        self._maxConcurrentSynchronizations = opts['maxConcurrentSynchronizations'] if 'maxConcurrentSynchronizations'\
+                                                                                       in opts else 10
+        retry_opts = opts['retryOpts'] if 'retryOpts' in opts else {}
+        self._retries = retry_opts['retries'] if 'retries' in opts else 5
+        self._minRetryDelayInSeconds = retry_opts['minDelayInSeconds'] if 'minDelayInSeconds' in retry_opts else 1
+        self._maxRetryDelayInSeconds = retry_opts['maxDelayInSeconds'] if 'maxDelayInSeconds' in retry_opts else 30
         self._token = token
         self._requestResolves = {}
         self._synchronizationListeners = {}
         self._latencyListeners = []
         self._connected = False
         self._socket = None
+        self._sessionId = None
         self._reconnectListeners = []
         self._connectedHosts = {}
         self._resubscriptionTriggerTimes = {}
+        self._synchronizationThrottler = SynchronizationThrottler(self, self._maxConcurrentSynchronizations)
+        self._synchronizationThrottler.start()
         self._packetOrderer = PacketOrderer(self, opts['packetOrderingTimeout'])
         if 'packetLogger' in opts and 'enabled' in opts['packetLogger'] and opts['packetLogger']['enabled']:
             self._packetLogger = PacketLogger(opts['packetLogger'])
@@ -91,13 +101,14 @@ class MetaApiWebsocketClient:
             self._connected = True
             self._requestResolves = {}
             self._resolved = False
-            result = asyncio.Future()
+            self._connectResult = asyncio.Future()
             self._packetOrderer.start()
             url = f'{self._url}?auth-token={self._token}'
             self._socket = socketio.AsyncClient(reconnection=False, request_timeout=self._request_timeout)
 
             while not self._socket.connected:
                 try:
+                    self._sessionId = random_id()
                     await asyncio.wait_for(
                         self._socket.connect(url, socketio_path='ws',
                                              headers={'Client-id': '{:01.10f}'.format(random())}),
@@ -110,7 +121,7 @@ class MetaApiWebsocketClient:
                 print(f'[{datetime.now().isoformat()}] MetaApi websocket client connected to the MetaApi server')
                 if not self._resolved:
                     self._resolved = True
-                    result.set_result(None)
+                    self._connectResult.set_result(None)
 
                 if not self._connected:
                     self._socket.disconnect()
@@ -120,17 +131,18 @@ class MetaApiWebsocketClient:
                 print(f'[{datetime.now().isoformat()}] MetaApi websocket client connection error', err)
                 if not self._resolved:
                     self._resolved = True
-                    result.set_exception(Exception(err))
+                    self._connectResult.set_exception(Exception(err))
 
             @self._socket.on('connect_timeout')
             def on_connect_timeout(timeout):
                 print(f'[{datetime.now().isoformat()}] MetaApi websocket client connection timeout')
                 if not self._resolved:
                     self._resolved = True
-                    result.set_exception(TimeoutException('MetaApi websocket client connection timed out'))
+                    self._connectResult.set_exception(TimeoutException('MetaApi websocket client connection timed out'))
 
             @self._socket.on('disconnect')
             async def on_disconnect():
+                self._synchronizationThrottler.on_disconnect()
                 print(f'[{datetime.now().isoformat()}] MetaApi websocket client disconnected from the MetaApi server')
                 await self._reconnect()
 
@@ -147,7 +159,8 @@ class MetaApiWebsocketClient:
                 else:
                     request_resolve = asyncio.Future()
                 self._convert_iso_time_to_date(data)
-                request_resolve.set_result(data)
+                if not request_resolve.done():
+                    request_resolve.set_result(data)
                 if 'timestamps' in data and hasattr(request_resolve, 'type'):
                     data['timestamps']['clientProcessingFinished'] = datetime.now()
                     for listener in self._latencyListeners:
@@ -165,7 +178,8 @@ class MetaApiWebsocketClient:
                 if data['requestId'] in self._requestResolves:
                     request_resolve = self._requestResolves[data['requestId']]
                     del self._requestResolves[data['requestId']]
-                    request_resolve.set_exception(self._convert_error(data))
+                    if not request_resolve.done():
+                        request_resolve.set_exception(self._convert_error(data))
 
             @self._socket.on('synchronization')
             async def on_synchronization(data):
@@ -174,7 +188,7 @@ class MetaApiWebsocketClient:
                 self._convert_iso_time_to_date(data)
                 await self._process_synchronization_packet(data)
 
-            return result
+            return self._connectResult
 
     async def close(self):
         """Closes connection to MetaApi server"""
@@ -478,10 +492,11 @@ class MetaApiWebsocketClient:
         Returns:
             A coroutine which resolves when synchronization is started.
         """
-        return self._rpc_request(account_id, {'requestId': synchronization_id, 'type': 'synchronize',
-                                              'startingHistoryOrderTime': format_date(starting_history_order_time),
-                                              'startingDealTime': format_date(starting_deal_time),
-                                              'instanceIndex': instance_index})
+        return self._synchronizationThrottler.schedule_synchronize(account_id, {
+            'requestId': synchronization_id, 'type': 'synchronize',
+            'startingHistoryOrderTime': format_date(starting_history_order_time),
+            'startingDealTime': format_date(starting_deal_time),
+            'instanceIndex': instance_index})
 
     def wait_synchronized(self, account_id: str, instance_index: int, application_pattern: str,
                           timeout_in_seconds: float):
@@ -642,12 +657,20 @@ class MetaApiWebsocketClient:
             try:
                 await self._socket.disconnect()
                 url = f'{self._url}?auth-token={self._token}'
-                await asyncio.wait_for(self._socket.connect(url, socketio_path='ws'), timeout=self._connect_timeout)
+                self._connectResult = asyncio.Future()
+                self._resolved = False
+                self._sessionId = random_id()
+                await asyncio.wait_for(self._socket.connect(url, socketio_path='ws',
+                                                            headers={'Client-id': '{:01.10f}'.format(random())}),
+                                       timeout=self._connect_timeout)
+                await self._connectResult
                 reconnected = True
                 await self._fire_reconnected()
                 await self._socket.wait()
             except Exception:
-                pass
+                self._connectResult.cancel()
+                self._connectResult = None
+                await asyncio.sleep(1)
 
     async def _rpc_request(self, account_id: str, request: dict, timeout_in_seconds: float = None) -> Coroutine:
         if not self._connected:
@@ -658,12 +681,30 @@ class MetaApiWebsocketClient:
         if not self._resolved:
             raise TimeoutException(f"MetaApi websocket client request of account {account_id} timed because socket "
                                    f"client failed to connect to the server.")
+        if request['type'] == 'subscribe':
+            request['sessionId'] = self._sessionId
+        if request['type'] in ['trade', 'subscribe']:
+            return await self._make_request(account_id, request, timeout_in_seconds)
+        retry_counter = 0
+        while True:
+            try:
+                return await self._make_request(account_id, request, timeout_in_seconds)
+            except Exception as err:
+                if err.__class__.__name__ in ['NotSynchronizedException', 'TimeoutException',
+                                              'NotAuthenticatedException', 'InternalException'] and retry_counter < \
+                        self._retries:
+                    await asyncio.sleep(min(pow(2, retry_counter) * self._minRetryDelayInSeconds,
+                                            self._maxRetryDelayInSeconds))
+                    retry_counter += 1
+                else:
+                    raise err
+
+    async def _make_request(self, account_id: str, request: dict, timeout_in_seconds: float = None):
         if 'requestId' in request:
             request_id = request['requestId']
         else:
             request_id = random_id()
             request['requestId'] = request_id
-
         request['timestamps'] = {'clientProcessingStarted': format_date(datetime.now())}
         self._requestResolves[request_id] = asyncio.Future()
         self._requestResolves[request_id].type = request['type']
@@ -725,25 +766,28 @@ class MetaApiWebsocketClient:
         try:
             packets = self._packetOrderer.restore_order(packet)
             for data in packets:
+                if 'synchronizationId' in data:
+                    self._synchronizationThrottler.update_synchronization_id(data['synchronizationId'])
                 instance_id = data['accountId'] + ':' + str(data['instanceIndex'] if 'instanceIndex' in data else 0)
                 instance_index = data['instanceIndex'] if 'instanceIndex' in data else 0
                 if data['type'] == 'authenticated':
-                    if 'host' in data:
-                        self._connectedHosts[instance_id] = data['host']
+                    if 'sessionId' not in data or data['sessionId'] == self._sessionId:
+                        if 'host' in data:
+                            self._connectedHosts[instance_id] = data['host']
 
-                    on_connected_tasks: List[asyncio.Task] = []
+                        on_connected_tasks: List[asyncio.Task] = []
 
-                    async def run_on_connected(listener):
-                        try:
-                            await listener.on_connected(instance_index, data['replicas'])
-                        except Exception as err:
-                            print(f'{data["accountId"]}: Failed to notify listener about connected event', err)
+                        async def run_on_connected(listener):
+                            try:
+                                await listener.on_connected(instance_index, data['replicas'])
+                            except Exception as err:
+                                print(f'{data["accountId"]}: Failed to notify listener about connected event', err)
 
-                    if data['accountId'] in self._synchronizationListeners:
-                        for listener in self._synchronizationListeners[data['accountId']]:
-                            on_connected_tasks.append(asyncio.create_task(run_on_connected(listener)))
-                    if len(on_connected_tasks) > 0:
-                        await asyncio.wait(on_connected_tasks)
+                        if data['accountId'] in self._synchronizationListeners:
+                            for listener in self._synchronizationListeners[data['accountId']]:
+                                on_connected_tasks.append(asyncio.create_task(run_on_connected(listener)))
+                        if len(on_connected_tasks) > 0:
+                            await asyncio.wait(on_connected_tasks)
                 elif data['type'] == 'disconnected':
                     if instance_id in self._connectedHosts and self._connectedHosts[instance_id] == data['host']:
                         on_disconnected_tasks: List[asyncio.Task] = []
@@ -989,6 +1033,7 @@ class MetaApiWebsocketClient:
                         on_deal_synchronization_finished_tasks: List[asyncio.Task] = []
 
                         async def run_on_deal_synchronization_finished(listener):
+                            self._synchronizationThrottler.remove_synchronization_id(data['synchronizationId'])
                             try:
                                 await listener.on_deal_synchronization_finished(instance_index,
                                                                                 data['synchronizationId'])
