@@ -13,6 +13,10 @@ class HistoryStorage(SynchronizationListener, ABC):
         self._orderSynchronizationFinished = {}
         self._dealSynchronizationFinished = {}
 
+    async def initialize(self):
+        """Initializes the storage and loads required data from a persistent storage."""
+        pass
+
     @property
     def order_synchronization_finished(self) -> bool:
         """Returns flag indicating whether order history synchronization has finished.
@@ -31,6 +35,11 @@ class HistoryStorage(SynchronizationListener, ABC):
         """
         return True in list(self._dealSynchronizationFinished.values())
 
+    @abstractmethod
+    async def clear(self):
+        """Clears the storage and deletes persistent data."""
+        pass
+
     @abstractmethod
     async def last_history_order_time(self, instance_index: int = None) -> datetime:
         """Returns the time of the last history order record stored in the history storage.
