@@ -77,7 +77,10 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         self._subscriptions = {}
         self._stateByInstanceIndex = {}
         self._synchronized = False
-        self._shouldSynchronize = None
+        self._shouldRetrySubscribe = False
+        self._isSubscribing = False
+        self._subscribeTask = None
+        self._subscribeFuture = None
 
     def get_account_information(self) -> 'Coroutine[asyncio.Future[MetatraderAccountInformation]]':
         """Returns account information (see
@@ -221,7 +224,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         Returns:
             A coroutine resolving when the history is cleared.
         """
-        self._historyStorage.reset()
+        asyncio.create_task(self._historyStorage.clear())
         return self._websocketClient.remove_history(self._account.id, application)
 
     def remove_application(self):
@@ -231,7 +234,7 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         Returns:
             A coroutine resolving when the history is cleared and application is removed.
         """
-        self._historyStorage.reset()
+        asyncio.create_task(self._historyStorage.clear())
         return self._websocketClient.remove_application(self._account.id)
 
     def create_market_buy_order(self, symbol: str, volume: float, stop_loss: float = None, take_profit: float = None,
@@ -625,19 +628,39 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
 
     async def initialize(self):
         """Initializes meta api connection"""
-        await self._historyStorage.load_data_from_disk()
+        await self._historyStorage.initialize()
 
-    async def subscribe(self) -> Coroutine:
+    async def subscribe(self):
         """Initiates subscription to MetaTrader terminal.
 
         Returns:
             A coroutine which resolves when subscription is initiated.
         """
-        try:
-            return await self._websocketClient.subscribe(self._account.id)
-        except Exception as err:
-            if err.__class__.__name__ != 'TimeoutException':
-                raise err
+        if not self._isSubscribing:
+            self._isSubscribing = True
+            self._shouldRetrySubscribe = True
+            subscribe_retry_interval_in_seconds = 3
+            while self._shouldRetrySubscribe and not self._closed:
+                try:
+                    await self._websocketClient.subscribe(self._account.id)
+                except Exception:
+                    pass
+                if not self._shouldRetrySubscribe:
+                    break
+                retry_interval = subscribe_retry_interval_in_seconds
+                subscribe_retry_interval_in_seconds = min(subscribe_retry_interval_in_seconds * 2, 300)
+                subscribe_future = asyncio.Future()
+
+                async def subscribe_task():
+                    await asyncio.sleep(retry_interval)
+                    subscribe_future.set_result(True)
+                self._subscribeTask = asyncio.create_task(subscribe_task())
+                self._subscribeFuture = subscribe_future
+                result = await self._subscribeFuture
+                self._subscribeFuture = None
+                if not result:
+                    break
+            self._isSubscribing = False
 
     def subscribe_to_market_data(self, symbol: str, instance_index: int = None) -> Coroutine:
         """Subscribes on market data of specified symbol (see
@@ -741,6 +764,10 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         Returns:
             A coroutine which resolves when the asynchronous event is processed.
         """
+        if self._subscribeFuture and not self._subscribeFuture.done():
+            self._subscribeFuture.set_result(False)
+            self._subscribeTask.cancel()
+        self._shouldRetrySubscribe = False
         key = random_id(32)
         state = self._get_state(instance_index)
         state['shouldSynchronize'] = key
@@ -783,7 +810,6 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         """
         state = self._get_state(instance_index)
         state['dealsSynchronized'][synchronization_id] = True
-        await self._historyStorage.update_disk_storage()
 
     async def on_order_synchronization_finished(self, instance_index: int, synchronization_id: str):
         """Invoked when a synchronization of history orders on a MetaTrader account have finished.
@@ -804,6 +830,9 @@ class MetaApiConnection(SynchronizationListener, ReconnectListener):
         Returns:
             A coroutine which resolves when connection to MetaApi websocket API restored after a disconnect.
         """
+        if self._subscribeFuture and not self._subscribeFuture.done():
+            self._subscribeFuture.set_result(False)
+            self._subscribeTask.cancel()
         await self.subscribe()
 
     async def is_synchronized(self, instance_index: int, synchronization_id: str = None) -> bool:
