@@ -3,6 +3,7 @@ from ..clients.metaApi.metaApiWebsocket_client import MetaApiWebsocketClient
 from .models import MetatraderHistoryOrders, MetatraderDeals
 from ..clients.metaApi.reconnectListener import ReconnectListener
 from ..clients.metaApi.synchronizationListener import SynchronizationListener
+from ..clients.timeoutException import TimeoutException
 from .metatraderAccount import MetatraderAccount
 from datetime import datetime, timedelta
 from mock import MagicMock, AsyncMock, patch
@@ -10,6 +11,7 @@ from .models import date
 from typing import Coroutine
 import pytest
 import asyncio
+from asyncio import sleep
 
 
 class MockClient(MetaApiWebsocketClient):
@@ -107,17 +109,28 @@ class AutoMockAccount(MetatraderAccount):
         return 'automatic'
 
 
-account = MockAccount(MagicMock(), MagicMock(), MagicMock(), MagicMock())
-auto_account = AutoMockAccount(MagicMock(), MagicMock(), MagicMock(), MagicMock())
-client = MockClient('token')
+account: MockAccount = None
+auto_account: AutoMockAccount = None
+client: MockClient = None
 api: MetaApiConnection = None
 
 
 @pytest.fixture(autouse=True)
 async def run_around_tests():
+    global account
+    account = MockAccount(MagicMock(), MagicMock(), MagicMock(), MagicMock())
+    global auto_account
+    auto_account = AutoMockAccount(MagicMock(), MagicMock(), MagicMock(), MagicMock())
+    global client
+    client = MockClient('token')
+    storage = MagicMock()
+    storage.last_history_order_time = AsyncMock(return_value=datetime.now())
+    storage.last_deal_time = AsyncMock(return_value=datetime.now())
     global api
-    api = MetaApiConnection(client, account, MagicMock(), MagicMock())
+    api = MetaApiConnection(client, account, storage, MagicMock())
     yield
+    api.health_monitor.stop()
+    client._synchronizationThrottler.stop()
 
 
 class TestMetaApiConnection:
@@ -408,7 +421,9 @@ class TestMetaApiConnection:
     async def test_remove_history(self):
         """Should remove history."""
         client.remove_history = AsyncMock()
+        api.history_storage.clear = AsyncMock()
         await api.remove_history('app')
+        api.history_storage.clear.assert_called()
         client.remove_history.assert_called_with('accountId', 'app')
 
     @pytest.mark.asyncio
@@ -653,9 +668,72 @@ class TestMetaApiConnection:
     async def test_subscribe_to_terminal(self):
         """Should subscribe to terminal."""
         client.subscribe = AsyncMock()
+
+        async def delay_connect():
+            await asyncio.sleep(0.1)
+            await api.on_connected(1, 1)
+
+        asyncio.create_task(delay_connect())
         await api.subscribe()
         client.subscribe.assert_called_with('accountId')
 
+    @pytest.mark.asyncio
+    async def test_retry_subscribe_to_terminal(self):
+        """Should retry subscribe to terminal if no response received."""
+        with patch('lib.metaApi.metaApiConnection.asyncio.sleep', new=lambda x: sleep(x / 10)):
+            response = {'type': 'response', 'accountId': 'accountId', 'requestId': 'requestId'}
+            client.subscribe = AsyncMock(side_effect=[TimeoutException('timeout'), response, response])
+
+            async def delay_connect():
+                await sleep(0.36)
+                await api.on_connected(1, 1)
+
+            asyncio.create_task(delay_connect())
+            await api.subscribe()
+            client.subscribe.assert_called_with('accountId')
+            assert client.subscribe.call_count == 2
+
+    @pytest.mark.asyncio
+    async def test_subscribe_after_reconnect(self):
+        """Should subscribe after reconnect."""
+        with patch('lib.metaApi.metaApiConnection.asyncio.sleep', new=lambda x: sleep(x / 10)):
+            client.connect = AsyncMock()
+            client.subscribe = AsyncMock()
+            asyncio.create_task(api.subscribe())
+            await sleep(0.18)
+            client.subscribe.assert_called_with('accountId')
+            await api.on_connected(1, 1)
+            asyncio.create_task(api.on_reconnected())
+            await sleep(0.33)
+            assert client.subscribe.call_count == 3
+            await api.on_connected(1, 1)
+            await sleep(0.18)
+            assert client.subscribe.call_count == 3
+
+    @pytest.mark.asyncio
+    async def test_no_multiple_subscribes(self):
+        """Should not send multiple subscribe requests at the same time."""
+        client.subscribe = AsyncMock()
+        asyncio.create_task(api.subscribe())
+        asyncio.create_task(api.subscribe())
+        await asyncio.sleep(0.5)
+        await api.on_connected(1, 1)
+        await asyncio.sleep(0.6)
+        client.subscribe.assert_called_with('accountId')
+        assert client.subscribe.call_count == 1
+
+    @pytest.mark.asyncio
+    async def test_not_retry_subscribe_after_close(self):
+        """Should not retry subscribe to terminal if connection is closed."""
+        client.subscribe = AsyncMock(side_effect=TimeoutException('timeout'))
+        client.unsubscribe = AsyncMock()
+        asyncio.create_task(api.subscribe())
+        asyncio.create_task(api.close())
+        await asyncio.sleep(1)
+        client.subscribe.assert_called_with('accountId')
+        assert client.subscribe.call_count == 1
+        client.unsubscribe.assert_called_with('accountId')
+
     @pytest.mark.asyncio
     async def test_synchronize_state_with_terminal(self):
         """Should synchronize state with terminal."""
@@ -827,7 +905,6 @@ class TestMetaApiConnection:
         await promise
         assert pytest.approx(10, 10) == (datetime.now() - start_time).seconds * 1000
         assert (await api.is_synchronized(1, 'synchronizationId'))
-        api._historyStorage.update_disk_storage.assert_called()
 
     @pytest.mark.asyncio
     async def test_time_out_waiting_for_sync(self):
@@ -850,6 +927,6 @@ class TestMetaApiConnection:
     @pytest.mark.asyncio
     async def test_load_history_storage_from_disk(self):
         """Should load data to history storage from disk."""
-        api._historyStorage.load_data_from_disk = AsyncMock()
+        api._historyStorage.initialize = AsyncMock()
         await api.initialize()
-        api._historyStorage.load_data_from_disk.assert_called()
+        api._historyStorage.initialize.assert_called()
